/*
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2002 The Ant-Contrib project.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Ant", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */
package net.sf.antcontrib.cpptasks;

import net.sf.antcontrib.cpptasks.borland.BorlandCCompiler;
import net.sf.antcontrib.cpptasks.borland.BorlandResourceCompiler;
import net.sf.antcontrib.cpptasks.compaq.CompaqVisualFortranCompiler;
import net.sf.antcontrib.cpptasks.compiler.Compiler;
import net.sf.antcontrib.cpptasks.devstudio.DevStudioCCompiler;
import net.sf.antcontrib.cpptasks.devstudio.DevStudioMIDLCompiler;
import net.sf.antcontrib.cpptasks.devstudio.DevStudioResourceCompiler;
import net.sf.antcontrib.cpptasks.gcc.GccCCompiler;
import net.sf.antcontrib.cpptasks.hp.aCCCompiler;
import net.sf.antcontrib.cpptasks.ibm.VisualAgeCCompiler;
import net.sf.antcontrib.cpptasks.intel.IntelLinux32CCompiler;
import net.sf.antcontrib.cpptasks.intel.IntelLinux64CCompiler;
import net.sf.antcontrib.cpptasks.intel.IntelWin32CCompiler;
import net.sf.antcontrib.cpptasks.intel.IntelWin64CCompiler;
import net.sf.antcontrib.cpptasks.os390.OS390CCompiler;
import net.sf.antcontrib.cpptasks.os400.IccCompiler;
import net.sf.antcontrib.cpptasks.sun.C89CCompiler;
import net.sf.antcontrib.cpptasks.sun.ForteCCCompiler;
import org.apache.tools.ant.types.EnumeratedAttribute;


/**
 *   Enumeration of supported compilers
 *
     * <table width="100%" border="1">
     *    <thead>Supported compilers</thead>
     *    <tr><td>gcc (default)</td><td>GCC C++ compiler</td></tr>
     *    <tr><td>g++</td><td>GCC C++ compiler</td></tr>
     *    <tr><td>c++</td><td>GCC C++ compiler</td></tr>
     *   <tr><td>msvc</td><td>Microsoft Visual C++</td></tr>
     *   <tr><td>bcc</td><td>Borland C++ Compiler</td></tr>
     *   <tr><td>msrc</td><td>Microsoft Resource Compiler</td></tr>
     *   <tr><td>brc</td><td>Borland Resource Compiler</td></tr>
     *   <tr><td>df</td><td>Compaq Visual Fortran Compiler</td></tr>
     *   <tr><td>midl</td><td>Microsoft MIDL Compiler</td></tr>
     *   <tr><td>icl</td><td>Intel C++ compiler for Windows (IA-32)</td></tr>
     *   <tr><td>ecl</td><td>Intel C++ compiler for Windows (IA-64)</td></tr>
     *   <tr><td>icc</td><td>Intel C++ compiler for Linux (IA-32)</td></tr>
     *   <tr><td>ecc</td><td>Intel C++ compiler for Linux (IA-64)</td></tr>
     *   <tr><td>CC</td><td>Sun ONE C++ compiler</td></tr>
     *   <tr><td>aCC</td><td>HP aC++ C++ Compiler</td></tr>
     *   <tr><td>os390</td><td>OS390 C Compiler</td></tr>
     *   <tr><td>os400</td><td>Icc Compiler</td></tr>
     *   <tr><td>sunc89</td><td>Sun C89 C Compiler</td></tr>
     *   <tr><td>xlC</td><td>VisualAge C Compiler</td></tr>
     * </table>
 *
 *   @author Curt Arnold
 *
 */
public class CompilerEnum extends EnumeratedAttribute {

    private final static ProcessorEnumValue[] compilers = new ProcessorEnumValue[] {
        new ProcessorEnumValue("gcc", GccCCompiler.getInstance()),
        new ProcessorEnumValue("g++", GccCCompiler.getGppInstance()),
        new ProcessorEnumValue("c++", GccCCompiler.getCppInstance()),
        new ProcessorEnumValue("msvc", DevStudioCCompiler.getInstance()),
        new ProcessorEnumValue("bcc", BorlandCCompiler.getInstance()),
        new ProcessorEnumValue("msrc", DevStudioResourceCompiler.getInstance()),
        new ProcessorEnumValue("brc", BorlandResourceCompiler.getInstance()),
        new ProcessorEnumValue("df", CompaqVisualFortranCompiler.getInstance()),
        new ProcessorEnumValue("midl", DevStudioMIDLCompiler.getInstance()),
        new ProcessorEnumValue("icl", IntelWin32CCompiler.getInstance()),
        new ProcessorEnumValue("ecl", IntelWin64CCompiler.getInstance()),
        new ProcessorEnumValue("icc", IntelLinux32CCompiler.getInstance()),
        new ProcessorEnumValue("ecc", IntelLinux64CCompiler.getInstance()),
        new ProcessorEnumValue("CC", ForteCCCompiler.getInstance()),
        new ProcessorEnumValue("aCC", aCCCompiler.getInstance()),
        new ProcessorEnumValue("os390", OS390CCompiler.getInstance()),
        new ProcessorEnumValue("os400", IccCompiler.getInstance()),
        new ProcessorEnumValue("sunc89", C89CCompiler.getInstance()),
        new ProcessorEnumValue("xlC", VisualAgeCCompiler.getInstance())
    };

    public String[] getValues() {
        return ProcessorEnumValue.getValues(compilers);
    }

    public Compiler getCompiler() {
        return (Compiler) compilers[getIndex()].getProcessor();
    }
}

