/*
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2002 The Ant-Contrib project.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Ant", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */
package net.sf.antcontrib.cpptasks;

import java.util.Vector;

/**
 *
 *
 * @author Curt Arnold
 */
public final class DependencyInfo {
    private /* final */ String source;
    private /* final */ long sourceLastModified;
    /**
     * Last modified time of this file or anything
     * that it depends on.
     *
     * Not persisted since almost any change
     * could invalidate it.  Initialized to long.MIN_VALUE
     * on construction.
     */
    private long compositeLastModified;
    private /* final */ String includePathIdentifier;
    private /* final */ String[] includes;
    private /* final */ String[] sysIncludes;

    public DependencyInfo(
      String includePathIdentifier,
      String source,
      long sourceLastModified,
      Vector includes,
      Vector sysIncludes) {
      if(source == null) {
        throw new NullPointerException("source");
      }
      if(includePathIdentifier == null) {
        throw new NullPointerException("includePathIdentifier");
      }
      this.source = source;
      this.sourceLastModified = sourceLastModified;
      this.includePathIdentifier = includePathIdentifier;
      this.includes = new String[includes.size()];
      if(includes.size() == 0) {
        compositeLastModified = sourceLastModified;
      }
      else {
        includes.copyInto(this.includes);
        compositeLastModified = Long.MIN_VALUE;
      }
      this.sysIncludes = new String[sysIncludes.size()];
      sysIncludes.copyInto(this.sysIncludes);
    }

    public String getSource() {
      return source;
    }

    public long getSourceLastModified() {
      return sourceLastModified;
    }

    public String[] getIncludes() {
        String[] includesClone = (String[]) includes.clone();
        return includesClone;
    }

    public String[] getSysIncludes() {
      String[] sysIncludesClone = (String[]) sysIncludes.clone();
      return sysIncludesClone;
    }


    /**
     * Returns the latest modification date of the source
     * or anything that it depends on.
     *
     * @returns the composite lastModified time, returns Long.MIN_VALUE if not set
     */
    public long getCompositeLastModified() {
      return compositeLastModified;
    }

    public void setCompositeLastModified(long lastMod) {
      compositeLastModified = lastMod;
    }

    public String getIncludePathIdentifier() {
      return includePathIdentifier;
    }

}
