/*
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2002 The Ant-Contrib project.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Ant", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */
package net.sf.antcontrib.cpptasks;

import java.io.File;
import java.util.Hashtable;
import java.util.Vector;

import net.sf.antcontrib.cpptasks.compiler.LinkerConfiguration;
import net.sf.antcontrib.cpptasks.compiler.ProcessorConfiguration;
import org.apache.tools.ant.BuildException;

/**
 * This class matches each visited file with an appropriate
 * compiler
 *
 * @author Curt Arnold
 */
public final class TargetMatcher implements FileVisitor
{
    private File outputDir;
    private ProcessorConfiguration[] processors;
    private Hashtable targets;
    private final File sourceFiles[] = new File[1];
    private CCTask task;
    private LinkerConfiguration linker;
    private Vector objectFiles;

    public TargetMatcher(CCTask task, File outputDir,
      ProcessorConfiguration[] processors,
      LinkerConfiguration linker,
      Vector objectFiles,
      Hashtable targets) {
      this.task = task;
      this.outputDir = outputDir;
      this.processors = processors;
      this.targets = targets;
      this.linker = linker;
      this.objectFiles = objectFiles;
    }

    public void visit(File parentDir,String filename) throws BuildException {
      //
      //   see if any processor wants to bid
      //       on this one
      ProcessorConfiguration selectedCompiler = null;
      int bid = 0;
      if(processors != null) {
        for(int k = 0; k < processors.length; k++) {
          int newBid = processors[k].bid(filename);
          if(newBid > bid) {
            bid = newBid;
            selectedCompiler = processors[k];
          }
        }
      }

      //
      //   no processor interested in file
      //      log diagnostic message
      if(bid <= 0) {
        if(linker != null) {
          int linkerbid = linker.bid(filename);
          if(linkerbid > 0) {
            File objFile = new File(parentDir,filename);
            objectFiles.addElement(objFile);
            if(linkerbid == 1) {
              task.log("Unrecognized file type " +
                objFile.toString() +
                " will be passed to linker");
            }
          }
        }
      } else {
        //
        //  get output file name
        //
        String outputFileName = selectedCompiler.getOutputFileName(filename);
        //
        //   if there is some output for this task
        //      (that is a source file and not an header file)
        //
        if(outputFileName != null) {
          sourceFiles[0] = new File(parentDir,filename);
        //
        //   see if the same output file has already been registered
        //
        TargetInfo previousTarget =
            (TargetInfo) targets.get(outputFileName);
        if(previousTarget == null) {
          targets.put(outputFileName,
            new TargetInfo(selectedCompiler,sourceFiles,null,
               new File(outputDir,outputFileName),
               selectedCompiler.getRebuild()));
        }
        else {
          if(!previousTarget.getSources()[0].equals(sourceFiles[0])) {
            StringBuffer builder = new StringBuffer("Output filename conflict: ");
            builder.append(outputFileName);
            builder.append(" would be produced from ");
            builder.append(previousTarget.getSources()[0].toString());
            builder.append(" and ");
            builder.append(filename);
            throw new BuildException(builder.toString());
          }
        }
      }
    }
  }

}
