/*
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2001 The Ant-Contrib project.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Ant", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */
package net.sf.antcontrib.cpptasks.compiler;

import java.io.File;

import net.sf.antcontrib.cpptasks.CCTask;
import net.sf.antcontrib.cpptasks.LinkerDef;
import net.sf.antcontrib.cpptasks.ProcessorDef;



/**
 * An abstract Linker implementation.
 *
 * @author Adam Murdoch
 */
public abstract class AbstractLinker extends AbstractProcessor implements Linker
{

    public AbstractLinker(String[] objExtensions, String[] ignoredExtensions) {
      super(objExtensions, ignoredExtensions);
    }

    abstract protected LinkerConfiguration createConfiguration(
      CCTask task,
      LinkType linkType,
      ProcessorDef[] baseConfigs,
      LinkerDef specificConfig);

    public ProcessorConfiguration createConfiguration(
        CCTask task,
        LinkType linkType,
        ProcessorDef[] baseConfigs, ProcessorDef specificConfig) {
        if(specificConfig == null) {
          throw new NullPointerException("specificConfig");
        }
        return createConfiguration(task,linkType,baseConfigs,
                (LinkerDef) specificConfig);
    }



    public abstract String getOutputFileName(String fileName);

    /**
     * Returns the bid of the processor for the file.
     *
     * A linker will bid 1 on any unrecognized file type.
     *
     * @param inputFile filename of input file
     * @return bid for the file, 0 indicates no interest, 1 indicates
     * that the processor recognizes the file but doesn't process it
     * (header files, for example), 100 indicates strong interest
     */
    public int bid(String inputFile) {
      int bid = super.bid(inputFile);
      switch(bid) {
        //
        //  unrecognized extension, take the file
        //
        case 0:
        return 1;

        //
        //   discard the ignored extensions
        //
        case 1:
        return 0;
      }
      return bid;
    }

    public String getLibraryKey(File libfile) {
        return libfile.getName();
    }

}
