/*
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2002 The Ant-Contrib project.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Ant", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */
package net.sf.antcontrib.cpptasks.compiler;


/**
 * An abstract processor (compiler/linker) implementation.
 *
 * @author Curt Arnold
 */
public abstract class AbstractProcessor implements Processor
{
    private /* final */ String[] sourceExtensions;
    private /* final */ String[] headerExtensions;


    protected AbstractProcessor(String[] sourceExtensions,
      String[] headerExtensions) {
      this.sourceExtensions = (String[]) sourceExtensions.clone();
      this.headerExtensions = (String[]) headerExtensions.clone();
    }

    abstract public String getIdentifier();

    public final String toString() {
      return getIdentifier();
    }


    /**
     * Returns the bid of the processor for the file.
     *
     * @param inputFile filename of input file
     * @return bid for the file, 0 indicates no interest, 1 indicates
     * that the processor recognizes the file but doesn't process it
     * (header files, for example), 100 indicates strong interest
     */
    public int bid(String inputFile) {
      String lower = inputFile.toLowerCase();
      for(int i = 0; i < sourceExtensions.length; i++) {
        if(lower.endsWith(sourceExtensions[i])) {
          return 100;
        }
      }
      for(int i = 0; i < headerExtensions.length; i++) {
        if(lower.endsWith(headerExtensions[i])) {
          return 1;
        }
      }
      return 0;
    }

    /**
     * Determines the identification of a command line processor by
     * capture the first line of its output for a specific command.
     *
     * @param command array of command line arguments starting with
     * executable name.  For example, { "cl" }
     * @param fallback start of identifier if there is an error in executing
     * the command
     * @return identifier for the processor
     */
    protected static String getIdentifier(String[] command, String fallback) {
        String identifier = fallback;
        try {
          String[] cmdout = CaptureStreamHandler.run(command);
          identifier = cmdout[0];
        }
        catch(Throwable ex) {
          identifier = fallback + ":" + ex.toString();
        }
        return identifier;
    }



}

