/*
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2002 The Ant-Contrib project.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Ant", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */
package net.sf.antcontrib.cpptasks.devstudio;

import java.io.File;
import java.util.Vector;

import net.sf.antcontrib.cpptasks.CUtil;
import net.sf.antcontrib.cpptasks.compiler.CommandLineCompilerConfiguration;
import net.sf.antcontrib.cpptasks.compiler.CompilerConfiguration;
import net.sf.antcontrib.cpptasks.compiler.LinkType;
import net.sf.antcontrib.cpptasks.compiler.PrecompilingCommandLineCCompiler;
import org.apache.tools.ant.BuildException;

/**
 * An abstract base class for compilers that are basically
 * command line compatible with Microsoft(r) C/C++ Optimizing Compiler
 *
 * @author Curt Arnold
 */
public abstract class DevStudioCompatibleCCompiler extends PrecompilingCommandLineCCompiler
{
    protected DevStudioCompatibleCCompiler(String command, String identifierArg) {
      super(command, identifierArg,
            new String[] { ".c", ".cc", ".cpp", ".cxx", ".c++" },
            new String[] { ".h", ".hpp", ".inl" }, ".obj", false, null );
    }


    protected File[] getEnvironmentIncludePath() {
        return CUtil.getPathFromEnvironment("INCLUDE",";");
    }

    protected String getIncludeDirSwitch(String includeDir) {
      return DevStudioProcessor.getIncludeDirSwitch(includeDir);
    }
    protected void addWarningSwitch(Vector args, int level) {
      DevStudioProcessor.addWarningSwitch(args,level);
    }

    protected void getDefineSwitch(StringBuffer buffer,String define, String value) {
      DevStudioProcessor.getDefineSwitch(buffer,define,value);
    }

    protected void getUndefineSwitch(StringBuffer buffer, String define) {
      DevStudioProcessor.getUndefineSwitch(buffer,define);
    }

    private static String[] mflags = new String[] {
      //
      //   first four are single-threaded
      //      (runtime=static,debug=false), (..,debug=true),
      //      (runtime=dynamic,debug=true), (..,debug=false), (not supported)
      //    next four are multi-threaded, same sequence
      "/ML", "/MLd", null, null, "/MT", "/MTd", "/MD", "/MDd" };

    protected void addImpliedArgs(Vector args,
      boolean debug,
      boolean multithreaded,
      boolean exceptions,
      LinkType linkType) {
      args.addElement("/c");
      args.addElement("/nologo");
      if(exceptions) {
        args.addElement("/GX");
      }
      int mindex = 0;
      if(multithreaded) {
        mindex += 4;
      }
      boolean staticRuntime = linkType.isStaticRuntime();
      if(!staticRuntime) {
        mindex += 2;
      }
      if(debug) {
        mindex += 1;
        args.addElement("/Zi");
        args.addElement("/Od");
        args.addElement("/GZ");
        args.addElement("/D_DEBUG");
      } else {
        args.addElement("/DNDEBUG");
      }
      String mflag = mflags[mindex];
      if(mflag == null) {
        throw new BuildException(
          "multithread='false' and runtime='dynamic' not supported");
      }
      args.addElement(mflag);

    }


   protected CompilerConfiguration createPrecompileGeneratingConfig(
      CommandLineCompilerConfiguration baseConfig, File prototype,
      String lastInclude) {
      String[] additionalArgs = new String[] {
          "/Fp" + CUtil.getBasename(prototype) + ".pch",
          "/Yc" };
      return new CommandLineCompilerConfiguration(baseConfig,
          additionalArgs, null, true);
   }

   protected CompilerConfiguration createPrecompileUsingConfig(
      CommandLineCompilerConfiguration baseConfig, File prototype,
      String lastInclude, String[] exceptFiles) {

      String[] additionalArgs = new String[] {
          "/Fp" + CUtil.getBasename(prototype) + ".pch",
          "/Yu" + lastInclude };
      return new CommandLineCompilerConfiguration(baseConfig,
          additionalArgs, exceptFiles, false);
   }


}
