/*
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2001 The Ant-Contrib project.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Ant", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */
package net.sf.antcontrib.cpptasks.gcc;

import net.sf.antcontrib.cpptasks.compiler.CommandLineLinker;
import net.sf.antcontrib.cpptasks.compiler.LinkType;

import java.io.File;
import java.util.*;

import org.apache.tools.ant.Project;
import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.types.FileSet;
import java.util.Hashtable;

import net.sf.antcontrib.cpptasks.CCTask;
import net.sf.antcontrib.cpptasks.LinkerDef;
import net.sf.antcontrib.cpptasks.compiler.LinkerConfiguration;
import net.sf.antcontrib.cpptasks.compiler.CommandLineLinkerConfiguration;
import net.sf.antcontrib.cpptasks.CUtil;
import net.sf.antcontrib.cpptasks.compiler.Linker;

/**
 * Adapter for the "ar" tool
 *
 * @author Adam Murdoch
 * @author Curt Arnold
 */
public abstract class AbstractArLibrarian extends CommandLineLinker
{
    protected AbstractArLibrarian(String command,
      String identificationArg,
      String[] inputExtensions,
      String[] ignoredExtensions,
      String outputExtension,
      boolean isLibtool, AbstractArLibrarian libtoolLibrarian)
    {
        super(command, identificationArg, inputExtensions,
          ignoredExtensions, outputExtension,
          isLibtool, libtoolLibrarian);
    }

    public File[] getLibraryPath() {
      return new File[0];
    }

    public void addImpliedArgs(boolean debug,LinkType linkType, Vector args) {
    }

    public String getCommandFileSwitch(String commandFile) {
      return null;
    }

    public String[] getOutputFileSwitch(String outputFile) {
      return GccProcessor.getOutputFileSwitch("rvs",outputFile);
    }

    public int getMaximumCommandLength() {
      return Integer.MAX_VALUE;
    }


    public void addStack(int stack, Vector args) {
    }

    public void addBase(long base, Vector args) {
    }

    public void addMap(boolean map, Vector args) {
    }

    public void addFixed(Boolean fixed, Vector args) {
    }

    public void addIncremental(boolean incremental, Vector args) {
    }

    public String[] getLibraryPatterns(String[] libnames) {
      return new String[0];
    }

    public boolean isCaseSensitive() {
      return true;
    }


    public void link(CCTask task,
                     File outputFile,
                     String[] sourceFiles,
                     CommandLineLinkerConfiguration config)
                     throws BuildException
    {
      //
      //   if there is an existing library then
      //      we must delete it before executing "ar"
      if(outputFile.exists()) {
          if(!outputFile.delete()) {
            throw new BuildException("Unable to delete " +
              outputFile.getAbsolutePath());
          }
      }
      //
      //   delegate to CommandLineLinker
      //
      super.link(task,outputFile,sourceFiles,config);
    }

}
