/*
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2001 The Ant-Contrib project.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Ant", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */
package net.sf.antcontrib.cpptasks.gcc;

import java.io.File;
import java.util.Vector;

import net.sf.antcontrib.cpptasks.CCTask;
import net.sf.antcontrib.cpptasks.CUtil;
import net.sf.antcontrib.cpptasks.compiler.CommandLineLinker;
import net.sf.antcontrib.cpptasks.compiler.CommandLineLinkerConfiguration;
import net.sf.antcontrib.cpptasks.compiler.LinkType;
import net.sf.antcontrib.cpptasks.types.LibrarySet;
import org.apache.tools.ant.Project;

/**
 * Abstract adapter for ld-like linkers
 *
 * @author Curt Arnold
 */
public abstract class AbstractLdLinker extends CommandLineLinker
{
    private String outputPrefix;

    protected AbstractLdLinker(String command,
        String identifierArg,
        String[] extensions,
        String[] ignoredExtensions,
        String outputPrefix,
        String outputSuffix,
        boolean isLibtool, AbstractLdLinker libtoolLinker) {
        super(command, identifierArg, extensions, ignoredExtensions, outputSuffix,
          isLibtool, libtoolLinker);
        this.outputPrefix = outputPrefix;
    }

    /**
     * Returns library path.
     *
     */
    public File[] getLibraryPath() {
      return new File[0];
    }

    public String getOutputFileName(String baseName) {
        return outputPrefix + super.getOutputFileName(baseName);
    }

    protected void addImpliedArgs(boolean debug, LinkType linkType, Vector args) {
      if(debug) {
        args.addElement("-g");
      }
      if(linkType.isStaticRuntime()) {
        args.addElement("-static");
      }
      if(linkType.isSharedLibrary()) {
        args.addElement("-shared");
      }
    }

    public String getCommandFileSwitch(String commandFile) {
      throw new IllegalStateException("ld does not support command files");
    }

    public String[] getOutputFileSwitch(String outputFile) {
      return GccProcessor.getOutputFileSwitch("-o",outputFile);
    }

    public int getMaximumCommandLength() {
      return Integer.MAX_VALUE;
    }

    public void addStack(int stack, Vector args) {
    }

    public void addIncremental(boolean incremental, Vector args) {
    }


    public void addBase(long base, Vector args) {
    }

    public void addMap(boolean map, Vector args) {
    }

    public void addFixed(Boolean fixed, Vector args) {
    }

    protected int addLibraryPatterns(
      String[] libnames,
      StringBuffer buf,
      String prefix,
      String extension,
      String[] patterns,
      int offset) {
      for(int i = 0; i < libnames.length; i++) {
        buf.setLength(0);
        buf.append(prefix);
        buf.append(libnames[i]);
        buf.append(extension);
        patterns[offset + i] = buf.toString();
      }
      return offset + libnames.length;
    }

    protected boolean isHPUX() {
      String osname = System.getProperty("os.name").toLowerCase();
      if(osname.indexOf("hp") >= 0 && osname.indexOf("ux") >= 0) {
        return true;
      }
      return false;
    }


    public String[] getLibraryPatterns(String[] libnames) {
      StringBuffer buf = new StringBuffer();
      String[] patterns = new String[libnames.length * 2];
      int offset = addLibraryPatterns(libnames,buf,
        "lib",".a",patterns,0);
      if(isHPUX()) {
        offset = addLibraryPatterns(libnames,buf,
          "lib",".sl",patterns,offset);
      } else {
        offset = addLibraryPatterns(libnames,buf,
          "lib",".so",patterns,offset);
      }
      return patterns;
    }

    public boolean isCaseSensitive() {
      return true;
    }

    public String[] addLibrarySets (CCTask task, LibrarySet[] libsets, Vector preargs,
        Vector midargs, Vector endargs) {
      Vector libnames = new Vector();
      super.addLibrarySets(task, libsets,preargs, midargs, endargs);
      StringBuffer buf = new StringBuffer("-l");
      for (int i = 0; i < libsets.length; i++) {
        LibrarySet set = libsets[i];
        File libdir = set.getDir(null);
        String[] libs = set.getLibs();
        if(libdir != null) {
          endargs.addElement("-L");
          endargs.addElement(libdir.getAbsolutePath());
        }
        for (int j = 0; j < libs.length; j++) {
          //
          //  reset the buffer to just "-l"
          //
          buf.setLength(2);
          //
          //  add the library name
          buf.append(libs[j]);
          libnames.addElement(libs[j]);
          //
          //  add the argument to the list
          endargs.addElement(buf.toString());
        }
      }
      
      String rc[] = new String[libnames.size()];
      for( int i=0; i < libnames.size(); i++ ) {
      	rc[i] = (String)libnames.elementAt(i);
      }
      return rc;
    }

    /**
     * Prepares argument list for exec command.  Will return null
     * if command line would exceed allowable command line buffer.
     *
     * @param outputFile linker output file
     * @param sourceFiles linker input files (.obj, .o, .res)
     * @param args linker arguments
     * @return arguments for runTask
     */
    public String[] prepareArguments(
        CCTask task,
        String outputDir,
        String outputFile,
        String[] sourceFiles,
        CommandLineLinkerConfiguration config) {
        	
        //
        //   need to suppress sources that correspond to
        //        library set entries since they are already
        //        in the argument list
        String[] libnames = config.getLibraryNames();
        if(libnames == null || libnames.length == 0) {
          return super.prepareArguments(task, outputDir,outputFile,sourceFiles,config);
        }
        //
        //
        //   null out any sources that correspond to library names
        //
        String[] localSources = (String[]) sourceFiles.clone();
        int extra = 0;
        for (int i = 0; i < libnames.length; i++) {
          String libname = libnames[i];
          for (int j = 0; j < localSources.length; j++) {
            if(localSources[j] != null &&
              localSources[j].indexOf(libname) > 0 &&
              localSources[j].indexOf("lib") > 0) {
              String filename = new File(localSources[j]).getName();
              if(filename.startsWith("lib") &&
                  filename.substring(3).startsWith(libname)) {
                  String extension = filename.substring(libname.length() + 3);
                  if(extension.equals(".a") ||
                    extension.equals(".so") ||
                    extension.equals(".sl")) {
                    localSources[j] = null;
                    extra++;
                 }
              }
            }
          }
        }

        if(extra == 0) {
          return super.prepareArguments(task, outputDir,outputFile,sourceFiles,config);
        }

        String[] finalSources = new String[localSources.length - extra];
        int index = 0;
        for (int i = 0; i < localSources.length; i++) {
          if (localSources[i] != null) {
            finalSources[index++] = localSources[i];
          }
        }
        return super.prepareArguments(task, outputDir,outputFile,finalSources,config);
    }

    public String getLibraryKey(File libfile) {
        String libname = libfile.getName();
        int lastDot = libname.lastIndexOf('.');
        if(lastDot >= 0) {
            return libname.substring(0,lastDot);
        }
        return libname;
    }

    /**
     * Returns library path.
     *
     */
      protected File[] getEnvironmentIncludePath() {
        return CUtil.getPathFromEnvironment("LIB", ":");
      }

}
