/*
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2001,2002 The Ant-Contrib project.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Ant", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */
package net.sf.antcontrib.cpptasks.types;

import net.sf.antcontrib.cpptasks.CUtil;
import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.Project;
import org.apache.tools.ant.types.AbstractFileSet;
import org.apache.tools.ant.types.FileSet;

/**
 * An Ant FileSet object augmented with if and unless conditions.
 *
 * @author Curt Arnold
 */
public class ConditionalFileSet extends FileSet
{
    private String ifCond;
    private String unlessCond;

    public ConditionalFileSet() {
    }

    /**
     *  Sets the property name for the 'if' condition.
     *
     *  The fileset will be ignored unless the property
     *    is defined.
     *
     *  The value of the property is insignificant, but values
     *      that would imply misinterpretation ("false", "no")
     *      will throw an exception when evaluated.
     */
    public void setIf(String propName)
    {
        ifCond = propName;
    }

    /**
     *  Set the property name for the 'unless' condition. 
     *
     *   If named property is set, the fileset will be ignored.
     *
     *   The value of the property is insignificant, but values
     *      that would imply misinterpretation ("false", "no")
     *      of the behavior will throw an exception when evaluated.
     *
     *  @param propName name of property
     */
    public void setUnless(String propName)
    {
        unlessCond = propName;
    }

    /**
     * Returns true if the Path's if and unless conditions
     * (if any) are satisfied.
     */
    public boolean isActive()
      throws BuildException {
        Project p = getProject();
        if (p == null) {
          throw new java.lang.IllegalStateException(
            "setProject() should have been called");
        }
        return CUtil.isActive(p,ifCond, unlessCond);
    }

    /**
     *   overrides FileSet's implementation which would
     *      throw an exception since the referenced object
     *      isn't this type.
     */
    protected AbstractFileSet getRef(Project p) {
        return (AbstractFileSet) ref.getReferencedObject(p);
    }

    public void execute() throws org.apache.tools.ant.BuildException
    {
        throw new org.apache.tools.ant.BuildException("Not an actual task, but looks like one for documentation purposes");
    }

}
